
addpath %add path to ISSM bin/ and lib/ directories

savingmodels = false;

%%% Choose task %%%
meshing = false;
param   = true;

%%% Choose run type %%%

%%% Saving directory %%%
dSaves = './Inputs/';

%%% Input for param %%%
paramInput   = [dSaves,'meshmdcoarsening.mat'];

%%% Fine md file %%%
pathGrISfine = [dSaves,'finalInitmd.mat']; 

%%% Output name %%%
outputMeshLoc  = [dSaves,'meshmdcoarsening.mat'];
outputParamLoc = [dSaves,'finalInitmdCoarsening.mat']; 
%%% Load %%%
mdfine     = loadmodel(pathGrISfine);

if(meshing && DefaultRun)    
    velcat     = [50,250]; %[m/yr]
    rsmin1     = 20000;
    rsmax1     = 25000;
    rsmin2     = 5000;
    rsmax2     = 15000;
    rsmin3     = 2000;
    rsmax3     = 5000;

    hmaxv        = NaN(size(mdfine.mesh.x));
    hminv        = NaN(size(mdfine.mesh.x));
    % Set min/max resolutions
    vel = mdfine.initialization.vel;
    pos = find(vel<velcat(1));
    hminv(pos) = rsmin1;
    hmaxv(pos) = rsmax1;
    pos = find(vel>=velcat(1) & vel<velcat(2));
    hminv(pos) = rsmin2;
    hmaxv(pos) = rsmax2;
    pos = find(vel>=velcat(2));
    hminv(pos) = rsmin3;
    hmaxv(pos) = rsmax3;

    md = bamg(mdfine,'field',vel,'err',17,'gradation',1.5,'hminVertices',hminv,'hmaxVertices',hmaxv);

    [md.mesh.lat,md.mesh.long] = xy2ll(md.mesh.x,md.mesh.y,+1,45,70);

    if savingmodels
        save(outputMeshLoc,'md','-v7.3');
    end
end

if(param)
    md = loadmodel(paramInput);
    %%% Projection %%%
    md.mesh.epsg = 3413;
    %%% Interpolation on coarsened mesh %%%
    md.geometry.thickness = InterpFromMeshToMesh2d(mdfine.mesh.elements,mdfine.mesh.x,mdfine.mesh.y,...
        mdfine.geometry.thickness,md.mesh.x,md.mesh.y);
    md.geometry.bed = InterpFromMeshToMesh2d(mdfine.mesh.elements,mdfine.mesh.x,mdfine.mesh.y,...
        mdfine.geometry.bed,md.mesh.x,md.mesh.y);
    md.geometry.base = InterpFromMeshToMesh2d(mdfine.mesh.elements,mdfine.mesh.x,mdfine.mesh.y,...
        mdfine.geometry.base,md.mesh.x,md.mesh.y);
    pos                  = find(md.geometry.base<md.geometry.bed);
    md.geometry.bed(pos) = md.geometry.base(pos);
    md.geometry.surface = md.geometry.base + md.geometry.thickness;
    md.mask.ice_levelset = InterpFromMeshToMesh2d(mdfine.mesh.elements,mdfine.mesh.x,mdfine.mesh.y,...
        mdfine.mask.ice_levelset,md.mesh.x,md.mesh.y);
    md.mask.ocean_levelset = InterpFromMeshToMesh2d(mdfine.mesh.elements,mdfine.mesh.x,mdfine.mesh.y,...
        mdfine.mask.ocean_levelset,md.mesh.x,md.mesh.y);
    pos                  = find(md.mask.ocean_levelset>0);
    md.geometry.bed(pos) = md.geometry.base(pos);
    md.initialization.vx = InterpFromMeshToMesh2d(mdfine.mesh.elements,mdfine.mesh.x,mdfine.mesh.y,...
        mdfine.initialization.vx,md.mesh.x,md.mesh.y);
    md.initialization.vy = InterpFromMeshToMesh2d(mdfine.mesh.elements,mdfine.mesh.x,mdfine.mesh.y,...
        mdfine.initialization.vy,md.mesh.x,md.mesh.y);
    md.initialization.vel = sqrt(md.initialization.vx.^2+md.initialization.vy.^2);
    if(DefaultRun)
        md.friction.coefficient = InterpFromMeshToMesh2d(mdfine.mesh.elements,mdfine.mesh.x,mdfine.mesh.y,...
            mdfine.friction.coefficient,md.mesh.x,md.mesh.y);
    elseif(SchoofRun)
       md.friction   = frictionschoof(); %set frictionschoof for the new md
       md.friction.C = InterpFromMeshToMesh2d(mdfine.mesh.elements,mdfine.mesh.x,mdfine.mesh.y,...
            mdfine.friction.C,md.mesh.x,md.mesh.y);
    end
    md.materials.rheology_B = InterpFromMeshToMesh2d(mdfine.mesh.elements,mdfine.mesh.x,mdfine.mesh.y,...
        mdfine.materials.rheology_B,md.mesh.x,md.mesh.y);
    md.initialization.pressure = InterpFromMeshToMesh2d(mdfine.mesh.elements,mdfine.mesh.x,mdfine.mesh.y,...
        mdfine.initialization.pressure,md.mesh.x,md.mesh.y);
    md.smb.mass_balance = InterpFromMeshToMesh2d(mdfine.mesh.elements,mdfine.mesh.x,mdfine.mesh.y,...
        mdfine.smb.mass_balance,md.mesh.x,md.mesh.y);
    
    %%% Various %%%
    md.basalforcings.groundedice_melting_rate = zeros(md.mesh.numberofvertices,1);
    md.basalforcings.floatingice_melting_rate = zeros(md.mesh.numberofvertices,1);
    if(DefaultRun)
        % Constant parameters %
        md.friction.p = mdfine.friction.p(1)*ones(md.mesh.numberofelements,1);
        md.friction.q = mdfine.friction.q(1)*ones(md.mesh.numberofelements,1);
    elseif(SchoofRun)
        % Constant parameters %
        md.friction.Cmax = mdfine.friction.Cmax(1)*ones(md.mesh.numberofvertices,1);
        md.friction.m    = mdfine.friction.m(1)*ones(md.mesh.numberofelements,1);
    end
    md.friction.effective_pressure = mdfine.friction.effective_pressure;
    md.friction.coupling = mdfine.friction.coupling;
    md.friction.effective_pressure_limit = mdfine.friction.effective_pressure_limit;
    md.materials.rheology_n = 3*ones(md.mesh.numberofelements,1);
    md.materials.rheology_law = 'Cuffey';
    md.stressbalance.referential = NaN*ones(md.mesh.numberofvertices,6);
    md.initialization.waterfraction = NaN;
    md.initialization.watercolumn   = NaN;    

    if savingmodels
        save(outputParamLoc,'md','-v7.3');
    end   
end











