

%%%%% Configure GrIS %%%%%
dGrIS    = './Inputs/';
dInputs  = dGrIS;
dSaves   = dGrIS;

savingmodels = false; %to save computed md

meshing            = false; %meshing work
param              = false; %parameterization
inversion1B        = false; %inversion for shelves-rheology
inversion1Fric     = false; %inversion for friction
thermal3dB         = false; %use 3D-thermal model for rheology
inversionFricFinal = false; %final inversion for friction coeff with rheology from 3D model

%%% Minimal ice thickness for friction regression %%%
lr_thickmin  = 500;
%%% Limit on ice thickness for B to be constrained %%%
adjB_limit   = 500;

figstart = 50;
%%%%% Meshing %%%%%
if(meshing)
    md = triangle(model,[dGrIS,'Greenland.exp'],10000);
    velcat     = [10,50,100,500]; %[m/yr]
    rsmin1     = 20000;
    rsmax1     = 25000;
    rsmin2     = 5000;
    rsmax2     = 15000;
    rsmin3     = 2000;
    rsmax3     = 5000;
    rsmin4     = 1000;
    rsmax4     = 2000;
    resRefined = 800;

    for ii=1:2 %execute twice to better capture small-scale features in 2nd iter
        % Velocities
        [velx vely]=interpJoughinmosaic250(md.mesh.x,md.mesh.y);
        vel=sqrt(velx.^2+vely.^2);
        vel(find(isnan(vel)))=0;
        % Thickness
        thickness = interpBedmachineGreenland(md.mesh.x,md.mesh.y,'thickness');
        hmaxv        = NaN(size(md.mesh.x));
        hminv        = NaN(size(md.mesh.x));
        % Set min/max resolutions
        pos = find(vel<velcat(1));
        hminv(pos) = rsmin1;
        hmaxv(pos) = rsmax1;
        pos = find(vel>=velcat(1) & vel<velcat(2));
        hminv(pos) = rsmin2;
        hmaxv(pos) = rsmax2;
        pos = find(vel>=velcat(2) & vel<velcat(3));
        hminv(pos) = rsmin3;
        hmaxv(pos) = rsmax3;
        pos = find(vel>=velcat(4));
        hminv(pos) = rsmin4;
        hmaxv(pos) = rsmax4;
        % Low resolution flags from Helene
        flags = ContourToNodes(md.mesh.x,md.mesh.y,[dGrIS,'NoRefinement.exp'],2);
        hminv(find(thickness<20 | flags)) = rsmin1; %nodes where we do not refine
        % Call BAMG
        md = bamg(md,'field',vel,'err',17,'gradation',1.3,'hminVertices',hminv,'hmaxVertices',hmaxv);
        md.private.bamg = struct();

        [velx vely]=interpJoughinmosaic250(md.mesh.x,md.mesh.y);
        vel=sqrt(velx.^2+vely.^2);
    end
    %%% Remeshing calving glaciers %%%
    % Velocities
    [velx vely]=interpJoughinmosaic250(md.mesh.x,md.mesh.y);
    vel=sqrt(velx.^2+vely.^2);
    vel(find(isnan(vel)))=0;
    % Thickness
    thickness = interpBedmachineGreenland(md.mesh.x,md.mesh.y,'thickness');
    hmaxv        = NaN(size(md.mesh.x));
    hminv        = NaN(size(md.mesh.x));
    % Set min/max resolutions
    pos = find(vel<velcat(1));
    hminv(pos) = rsmin1;
    hmaxv(pos) = rsmax1;
    pos = find(vel>=velcat(1) & vel<velcat(2));
    hminv(pos) = rsmin2;
    hmaxv(pos) = rsmax2;
    pos = find(vel>=velcat(2) & vel<velcat(3));
    hminv(pos) = rsmin3;
    hmaxv(pos) = rsmax3;
    pos = find(vel>=velcat(4));
    hminv(pos) = rsmin4;
    hmaxv(pos) = rsmax4;
    % Low resolution flags from Helene
    flags        = ContourToNodes(md.mesh.x,md.mesh.y,[dGrIS,'NoRefinement.exp'],2);
    hminv(find(thickness<20 | flags)) = rsmin1; %nodes where we do not refine
    % Refinement of calving glaciers
    flagsRefined = getExpRefinedGlaciers(md.mesh.x,md.mesh.y);
    hmaxv(flagsRefined) = resRefined;
    hminv(flagsRefined) = resRefined;
    % Re-meshing
    md = bamg(md,'field',vel,'err',17,'gradation',1.3,'hminVertices',hminv,'hmaxVertices',hmaxv);
    md.private.bamg = struct();
    if savingmodels
        save([dSaves,'meshmd.mat'],'md','-v7.3');
    end

end

if(param==true)
    md = loadmodel([dSaves,'meshmd.mat']);
    % Coordinates
    [md.mesh.lat,md.mesh.long] = xy2ll(md.mesh.x,md.mesh.y,+1,45,70);
    md.mesh.epsg = 3413;
    % Setup
    md.miscellaneous.name = 'setupGrISv1';
    md = setflowequation(md,'SSA','all');

    %%% From parameterization file %%%
    % Geometry from BedMachine
    md.geometry.bed       = interpBedmachineGreenland(md.mesh.x,md.mesh.y,'bed');
    md.geometry.surface   = interpBedmachineGreenland(md.mesh.x,md.mesh.y,'surface');
    md.geometry.thickness = interpBedmachineGreenland(md.mesh.x,md.mesh.y,'thickness');
    md.geometry.base      = md.geometry.surface-md.geometry.thickness;
    % Correct surface
    md.geometry.surface   = max(md.geometry.surface,1);
    md.geometry.thickness = md.geometry.surface-md.geometry.base;
    % Adjust ice thickness
    pos                        = find(md.geometry.thickness<=10);
    md.geometry.thickness(pos) = 10;
    md.geometry.base           = md.geometry.surface-md.geometry.thickness;
    pos                        = find(md.geometry.base<md.geometry.bed);
    md.geometry.bed(pos)       = md.geometry.base(pos);
    % Mask from BedMachine
    mask = int8(interpBedmachineGreenland(md.mesh.x,md.mesh.y,'mask'));
    md.mask.ice_levelset         = -1*ones(md.mesh.numberofvertices,1);
    md.mask.ocean_levelset = 1*ones(md.mesh.numberofvertices,1);
    pos  = find(mask<1); %find ocean vertices
    md.mask.ice_levelset(pos) = 1;
    pos = find(mask==0 | mask==3); %ocean and floating ice vertices
    md.mask.ocean_levelset(pos) = -1;
    pos = find(md.mesh.vertexonboundary);
    md.mask.ice_levelset(pos) = 1;
    % Ice shelf base based on hydrostatic equilibrium
    floating_base          = md.materials.rho_ice/(md.materials.rho_ice-md.materials.rho_water)*md.geometry.surface;
    pos                    = find(floating_base>md.geometry.bed);
    md.geometry.base(pos)  = floating_base(pos);
    di                     = md.materials.rho_ice/md.materials.rho_water;
    md.geometry.thickness  = md.geometry.surface-md.geometry.base;
    md.mask.ocean_levelset = md.geometry.thickness+md.geometry.bed/di;
    pos = find(md.geometry.base<md.geometry.bed | md.mask.ocean_levelset>0);
    md.geometry.bed(pos)   = md.geometry.base(pos);
    % Adjust ice mask
    pos = find(max(md.mask.ice_levelset(md.mesh.elements),[],2)>0); %offset mask by 1 elem to avoid cliff at transition
    md.mask.ice_levelset(md.mesh.elements(pos,:)) = 1;
    % Get velocities
    [velx vely] = interpJoughinmosaic250(md.mesh.x,md.mesh.y); %re-interpolate on new mesh
    vel                   = sqrt(velx.^2+vely.^2);
    vel(find(isnan(vel))) = 0;
    md.initialization.vx  = velx;
    md.initialization.vy  = vely;
    md.initialization.vel = vel;
    md.inversion.vx_obs   = velx;
    md.inversion.vy_obs   = vely;
    md.inversion.vel_obs  = vel;
    % Extra parameters
    md.basalforcings.groundedice_melting_rate = zeros(md.mesh.numberofvertices,1);
    md.basalforcings.floatingice_melting_rate = zeros(md.mesh.numberofvertices,1);

    % Slopes
    [sx,sy,s]=slope(md); %slope of each element
    sslope=averaging(md,s,10);
    % Extrapolate velocities to nodes with vel==0 for friction inference
    flags=(vel==0);
    pos1=find(flags);
    pos2=find(~flags);
    vel(pos1) = griddata(md.mesh.x(pos2),md.mesh.y(pos2),vel(pos2),md.mesh.x(pos1),md.mesh.y(pos1));
    vel=max(vel,0.1); %set minimum vel
    % Effective pressure
    Neff = md.materials.rho_ice*md.geometry.thickness+md.materials.rho_water*md.geometry.base;
    Neff(find(Neff<=0))=1;
    % Infer friction coefficient: drag compensates driving stress
    md.friction.coefficient=sqrt(md.materials.rho_ice*md.geometry.thickness.*(sslope)./(Neff.*vel/md.constants.yts));
    md.friction.coefficient=min(md.friction.coefficient,400);
    md.friction.p = ones(md.mesh.numberofelements,1);
    md.friction.q = ones(md.mesh.numberofelements,1);
    % Extrapolate friction coefficient on ice free regions
    flags=(md.mask.ice_levelset>0);
    pos1=find(flags);
    pos2=find(~flags);
    md.friction.coefficient(pos1) = griddata(md.mesh.x(pos2),md.mesh.y(pos2),md.friction.coefficient(pos2),md.mesh.x(pos1),md.mesh.y(pos1));
    pos=find(isnan(md.friction.coefficient));
    md.friction.coefficient(pos)  = 1;
    % Rheology
    md.materials.rheology_n = 3*ones(md.mesh.numberofelements,1);
    md.materials.rheology_B = cuffey(273.15 - 10)*ones(md.mesh.numberofvertices,1);
    md.materials.rheology_law = 'Cuffey';
    % SMB
    smbpath = [dGrIS,'smb_1961-1990.mat'];
    load(smbpath);
    md.smb.mass_balance = InterpFromGridToMesh(x_m,y_m,accumulation,md.mesh.x,md.mesh.y,-99999);
    pos = find(md.smb.mass_balance==-99999); %at southern tip
    md.smb.mass_balance(pos) = 0.0; %make sure we do that (07Feb2022) (does not really matter because we use SMBautoregression after init)
    % BCs
    md.stressbalance.spcvx = NaN*ones(md.mesh.numberofvertices,1);
    md.stressbalance.spcvy = NaN*ones(md.mesh.numberofvertices,1);
    md.stressbalance.spcvz = NaN*ones(md.mesh.numberofvertices,1); % 2D run
    md.stressbalance.referential  = NaN*ones(md.mesh.numberofvertices,6); %normal to bed slope
    md.stressbalance.loadingforce = 0*ones(md.mesh.numberofvertices,3); %changes to gravity
    pos = find((md.mask.ice_levelset<0).*(md.mesh.vertexonboundary)); %ice-covered vertices at domain boundaries (none)
    md.stressbalance.spcvx(pos) = md.initialization.vx(pos);
    md.stressbalance.spcvy(pos) = md.initialization.vy(pos);
    % Thermal
    md.basalforcings.geothermalflux = interpSeaRISE(md.mesh.x,md.mesh.y,'bheatflx');
    md.initialization.temperature   = min(0,interpSeaRISE(md.mesh.x,md.mesh.y,'surftemp'))+273.15;
    md.initialization.waterfraction = zeros(md.mesh.numberofvertices,1);
    md.initialization.watercolumn   = zeros(md.mesh.numberofvertices,1);
    md.thermal.spctemperature       = md.initialization.temperature;
    md.thermal.isenthalpy           = 1;
    md.thermal.isdynamicbasalspc    = 1;

    if savingmodels
        save([dSaves,'parammd.mat'],'md','-v7.3');
    end
    
end

if inversion1B
    md = loadmodel([dSaves,'parammd.mat']);
    %%%%% Inversion for rheology %%%%%
    % Tolerances
    md.stressbalance.restol=0.01;
    md.stressbalance.reltol=0.1;
    md.stressbalance.abstol=NaN;
    % Inversion setup
    md.inversion                    = m1qn3inversion(md.inversion);
    md.inversion.iscontrol          = 1 ;
    md.inversion.control_parameters = {'MaterialsRheologyBbar'};
    % Cost functions
    md.inversion.cost_functions = [101 103 502]; %AbsVel,LogVel,BbarGrad
    md.inversion.cost_functions_coefficients      = ones(md.mesh.numberofvertices,2);
    md.inversion.cost_functions_coefficients(:,1) = 2000;
    md.inversion.cost_functions_coefficients(:,2) = 60;
    md.inversion.cost_functions_coefficients(:,3) = 1e-17; %VV lowered to decrease contrib
    pos = find(md.inversion.vel_obs==0);
    md.inversion.cost_functions_coefficients(pos,1) = 0;
    md.inversion.cost_functions_coefficients(pos,2) = 0;
    % Bounds
    md.inversion.min_parameters = paterson(273.15-1)*ones(md.mesh.numberofvertices,1);
    md.inversion.max_parameters = paterson(273.15-50)*ones(md.mesh.numberofvertices,1);
    % Solve
    md.verbose               = verbose('solution',false,'control',true);
    md.cluster               = generic('np',1);
    mds                      = extract(md,md.mask.ocean_levelset<0); %extract ice shelves
    mds.stressbalance.restol = 0.003; %mds here (typo in original)
    mds                      = solve(mds,'sb');
    md.materials.rheology_B(mds.mesh.extractedvertices) = mds.results.StressbalanceSolution.MaterialsRheologyBbar;
    clear mds   
    if savingmodels
        save([dSaves,'inversion1Bmd.mat'],'md','-v7.3');
    end
end
if(inversion1Fric) 
    md = loadmodel([dSaves,'inversion1Bmd.mat']);
    md.cluster               = generic('name',oshostname(),'np',18); %cluster for runs
    md.cluster.executionpath = '';
    md.cluster.etcpath       = '';
     %%%%% Inversion for friction %%%%%
    % Add ice patches around ice shelves in ice mask
    pos = find(ContourToNodes(md.mesh.x,md.mesh.y,[dInputs,'IceAroundShelves.exp'],2));
    md.mask.ice_levelset(pos) = -1;
    % Inversion setup
    md.inversion                    = m1qn3inversion(md.inversion);
    md.inversion.iscontrol          = 1 ;
    md.inversion.control_parameters = {'FrictionCoefficient'};
    % Cost functions
    md.inversion.cost_functions = [101 103 501]; %AbsVel,LogVel,FrictCoeffGrad
    md.inversion.cost_functions_coefficients      = ones(md.mesh.numberofvertices,2);
    md.inversion.cost_functions_coefficients(:,1) = 2000;
    md.inversion.cost_functions_coefficients(:,2) = 4;
    md.inversion.cost_functions_coefficients(:,3) = 0.2*50^(-3);
    pos = find(md.mask.ice_levelset>0 | md.inversion.vel_obs==0);
    md.inversion.cost_functions_coefficients(pos,1) = 0;
    md.inversion.cost_functions_coefficients(pos,2) = 0;
    % Bounds
    md.inversion.maxsteps                = 110;
    md.inversion.maxiter                 = 110;
    md.inversion.min_parameters          = 0.05*ones(md.mesh.numberofvertices,1);
    md.inversion.max_parameters          = 400*ones(md.mesh.numberofvertices,1);
    md.inversion.control_scaling_factors = 1; %??
    % GL migration
    md.groundingline.migration = 'SubelementMigration';
    % Solve
    md.verbose = verbose('solution',false,'control',true);
    md         = solve(md,'sb');
    tfric = md.results.StressbalanceSolution.FrictionCoefficient;
    % Adjust friction coeff in ungrounded and low-thickness areas with linear regression on bed
    grice   = intersect(find(md.mask.ocean_levelset>0),find(md.geometry.thickness>=lr_thickmin));
    nogrice = [];
    for(ii=1:md.mesh.numberofvertices)
        if(any(grice==ii)==false)
            nogrice(end+1) = ii;
        end
    end
    nogrice = nogrice';
    linregrmod = fitlm(md.geometry.bed(grice),md.friction.coefficient(grice));
    tfric(nogrice) = linregrmod.predict(md.geometry.bed(nogrice));
    md.friction.coefficient = tfric;
    if savingmodels
        save([dSaves,'inversion1BFmd.mat'],'md','-v7.3');
    end
 
end

if thermal3dB
    md = loadmodel([dSaves,'inversion1BFmd.mat']);
    %%%%% 3D model %%%%%
    % Update velocities
    md.initialization.vx  = md.results.StressbalanceSolution.Vx;
    md.initialization.vy  = md.results.StressbalanceSolution.Vy;
    md.initialization.vel = md.results.StressbalanceSolution.Vel;
    % SPC velocities on inflow boundary
    pos_e = find(min(md.mask.ice_levelset(md.mesh.elements),[],2)<0); %ind of elems with at least one ice-covered node
    flags = zeros(md.mesh.numberofvertices,1);
    flags(md.mesh.elements(pos_e,:)) = 1;
    md2   = extract(md,flags); %model with ice-covered elements
    pos   = find(md2.mesh.vertexonboundary & ~outflow(md2)); %find inflow boundaries
    md.stressbalance.spcvx(md2.mesh.extractedvertices(pos)) = 0;
    md.initialization.vx(md2.mesh.extractedvertices(pos))   = 0;
    md.stressbalance.spcvy(md2.mesh.extractedvertices(pos)) = 0;
    md.initialization.vy(md2.mesh.extractedvertices(pos))   = 0;
    md.initialization.vel = sqrt(md.initialization.vx.^2+md.initialization.vy.^2);
    clear md2;
    % Extrusion
    md3d = md;
    %md = extrude(md,14,1.1);
    md3d = extrude(md3d,4,1.3);
    md3d = setflowequation(md3d,'HO','all');
    % 3D stress-balance
    md3d.inversion.iscontrol     = 0;
    md3d.cluster                 = generic('np',1);
    md3d                         = solve(md3d,'sb');
    md3d.initialization.vx       = md3d.results.StressbalanceSolution.Vx;
    md3d.initialization.vy       = md3d.results.StressbalanceSolution.Vy;
    md3d.initialization.vz       = md3d.results.StressbalanceSolution.Vz;
    md3d.initialization.vel      = md3d.results.StressbalanceSolution.Vel;
    md3d.initialization.pressure = md3d.results.StressbalanceSolution.Pressure;
    %%%%% Thermal model %%%%%
    % Steady-State params
    md3d.timestepping.time_step = 0;
    md3d.inversion.iscontrol    = 0;
    md3d.thermal.isenthalpy     = 1;
    % SPC
    pos = find(ContourToNodes(md3d.mesh.x,md3d.mesh.y,[dGrIS,'ThermalSpc.exp'],2) & md3d.mask.ice_levelset>0);
    md3d.thermal.spctemperature(pos) = 250;
    % Solve
    md3d.cluster = generic('np',1);
    md3d = solve(md3d,'thermal');
    md3d.initialization.temperature = md3d.results.ThermalSolution.Temperature;
    nv2d    = md.mesh.numberofvertices;
    % Compute depth-averaged B or B from depth-averaged temperature
    tempavg = zeros(nv2d,1);
    bCuf_fromTavg  = zeros(nv2d,1);
    bCuf_directavg = zeros(nv2d,1);
    for ii=1:nv2d
        deltah = zeros(md3d.mesh.numberoflayers-1,1);
        temp   = zeros(md3d.mesh.numberoflayers-1,1);
        for jj=1:(md3d.mesh.numberoflayers-1)
            deltah(jj) = md3d.mesh.z(ii+jj*nv2d)-md3d.mesh.z(ii+(jj-1)*nv2d);
            temp(jj)   = 1/2*(md3d.initialization.temperature(ii+jj*nv2d)+md3d.initialization.temperature(ii+(jj-1)*nv2d));
        end
        tempavg(ii) = (temp')*deltah/sum(deltah);
        bCuf_fromTavg(ii)  = (cuffey(temp)')*deltah/sum(deltah);
        bCuf_directavg(ii) = cuffey(tempavg(ii));
    end
    % Keep ice shelves rheology from inversion
    pos = find(md.mask.ocean_levelset<=0);
    bCuf_directavg(pos) = md.materials.rheology_B(pos);
    bCuf_fromTavg(pos)  = md.materials.rheology_B(pos);
    % Set rheology in 2D model
    tB = bCuf_directavg;
    % Adjust B in areas of thin ice
    ithin  = find(md.geometry.thickness<adjB_limit);
    ithick = find(md.geometry.thickness>=adjB_limit);
    tB(ithin) = min(max(tB(ithin),min(tB(ithick))),max(tB(ithick)));
    % Keep ice shelves rheology from inversion
    pos     = find(md.mask.ocean_levelset<=0);
    tB(pos) = md.materials.rheology_B(pos);
    md.materials.rheology_B = tB;
    % Adjust initialization fields in 2D model
    md.initialization.temperature = tempavg;
    md.initialization.vx  = md3d.results.StressbalanceSolution.Vx(end+1-md.mesh.numberofvertices:end);
    md.initialization.vy  = md3d.results.StressbalanceSolution.Vy(end+1-md.mesh.numberofvertices:end);
    md.initialization.vel = md3d.results.StressbalanceSolution.Vel(end+1-md.mesh.numberofvertices:end);
    md.initialization.pressure = md3d.results.StressbalanceSolution.Pressure(1:md.mesh.numberofvertices);
    if savingmodels
        save([dSaves,'thermalBmd.mat'],'md','-v7.3');
    end
end

if inversionFricFinal
    md = loadmodel([dSaves,'thermalBmd.mat']);
    %Tolerances
    md.stressbalance.restol = 0.01;
    md.stressbalance.reltol = 0.1;
    md.stressbalance.abstol = NaN;
    % Inversion setup
    md.inversion                    = m1qn3inversion(md.inversion);
    md.inversion.iscontrol          = 1 ;
    md.inversion.control_parameters = {'FrictionCoefficient'};
    % Cost functions
    md.inversion.cost_functions = [101 103 501]; %AbsVel,LogVel,FrictCoeffGrad
    md.inversion.cost_functions_coefficients      = ones(md.mesh.numberofvertices,2);
    md.inversion.cost_functions_coefficients(:,1) = 2000;
    md.inversion.cost_functions_coefficients(:,2) = 4; %VV lowered because higher contribution in misfit
    md.inversion.cost_functions_coefficients(:,3) = 20*50^(-3);
    pos = find(md.mask.ice_levelset>0 | md.inversion.vel_obs==0);
    md.inversion.cost_functions_coefficients(pos,1) = 0;
    md.inversion.cost_functions_coefficients(pos,2) = 0;
    % Bounds
    md.inversion.maxsteps                = 110;
    md.inversion.maxiter                 = 110;
    md.inversion.min_parameters          = 0.05*ones(md.mesh.numberofvertices,1);
    md.inversion.max_parameters          = 400*ones(md.mesh.numberofvertices,1);
    md.inversion.control_scaling_factors = 1; %??
    % GL migration
    md.groundingline.migration = 'SubelementMigration';
    % Solve
    md.verbose = verbose('solution',false,'control',true);
    md.cluster = generic('np',1);
    md         = solve(md,'sb');
    tfric = md.results.StressbalanceSolution.FrictionCoefficient;
    % Adjust friction coeff in ungrounded and low-thickness areas with linear regression on bed
    grice   = intersect(find(md.mask.ocean_levelset>0),find(md.geometry.thickness>=lr_thickmin));
    nogrice = [];
    for(ii=1:md.mesh.numberofvertices)
        if(any(grice==ii)==false)
            nogrice(end+1) = ii;
        end
    end
    nogrice = nogrice';
    linregrmod = fitlm(md.geometry.bed(grice),md.friction.coefficient(grice));
    tfric(nogrice) = linregrmod.predict(md.geometry.bed(nogrice));
    % Update friction field
    md.friction.coefficient = tfric;
    % Adjust initialization fields
    md.initialization.vx  = md.results.StressbalanceSolution.Vx;
    md.initialization.vy  = md.results.StressbalanceSolution.Vy;
    md.initialization.vel = md.results.StressbalanceSolution.Vel;
    md.initialization.pressure = md.results.StressbalanceSolution.Pressure;
    % md should be ready for solve('Transient') runs
    if savingmodels
        save([dSaves,'finalInitmd.mat'],'md','-v7.3');
    end
end


    
