
addpath %add path to ISSM bin/ and lib/ directories

%%%%% Configure GrIS mesh after the Coarse Spin-Up phase %%%%%
dInputs = './Inputs/';

ResCoarseSpin     = [dInputs,'myCoarseSpinp6pc1_dt7tm30000.mat'];
InitFine          = [dInputs,'finalInitmd22Feb2022.mat'];
ExpFile           = './Inputs/Greenland.exp';
BasinsRefinedGrid = [dInputs,'gridrefinement29April2022basinsGrISzwally.csv'];

savingmodels  = false; %to save computed md
meshing       = false; %meshing work
param         = true; %parameterization

mdcoarse   = loadmodel(ResCoarseSpin);
mdinitfine = loadmodel(InitFine);
if(meshing)
    md       = triangle(model,ExpFile,10000);

    velx = InterpFromMeshToMesh2d(mdcoarse.mesh.elements,mdcoarse.mesh.x,mdcoarse.mesh.y,...
        mdcoarse.results.TransientSolution(end).Vx,md.mesh.x,md.mesh.y);
    vely = InterpFromMeshToMesh2d(mdcoarse.mesh.elements,mdcoarse.mesh.x,mdcoarse.mesh.y,...
        mdcoarse.results.TransientSolution(end).Vy,md.mesh.x,md.mesh.y);
    vel  = sqrt(velx.^2+vely.^2);

    velcat     = [10,50,100,500]; %[m/yr]
    rsmin1     = 20000;
    rsmax1     = 25000;
    rsmin2     = 5000;
    rsmax2     = 15000;
    rsmin3     = 2000;
    rsmax3     = 5000;
    rsmin4     = 1000;
    rsmax4     = 2000;
    resRefined = 800;

    for ii=1:2 %execute twice to better capture small-scale features in 2nd iter
            % Velocities
            velx = InterpFromMeshToMesh2d(mdcoarse.mesh.elements,mdcoarse.mesh.x,mdcoarse.mesh.y,...
                mdcoarse.results.TransientSolution(end).Vx,md.mesh.x,md.mesh.y);
            vely = InterpFromMeshToMesh2d(mdcoarse.mesh.elements,mdcoarse.mesh.x,mdcoarse.mesh.y,...
                mdcoarse.results.TransientSolution(end).Vy,md.mesh.x,md.mesh.y);
            vel  = sqrt(velx.^2+vely.^2);
            % Thickness
            thickness = InterpFromMeshToMesh2d(mdcoarse.mesh.elements,mdcoarse.mesh.x,mdcoarse.mesh.y,...
                mdcoarse.results.TransientSolution(end).Thickness,md.mesh.x,md.mesh.y);
            hmaxv        = NaN(size(md.mesh.x));
            hminv        = NaN(size(md.mesh.x));
            % Set min/max resolutions
            pos = find(vel<velcat(1));
            hminv(pos) = rsmin1;
            hmaxv(pos) = rsmax1;
            pos = find(vel>=velcat(1) & vel<velcat(2));
            hminv(pos) = rsmin2;
            hmaxv(pos) = rsmax2;
            pos = find(vel>=velcat(2) & vel<velcat(3));
            hminv(pos) = rsmin3;
            hmaxv(pos) = rsmax3;
            pos = find(vel>=velcat(4));
            hminv(pos) = rsmin4;
            hmaxv(pos) = rsmax4;
            % Low resolution flags from Helene
            flags = ContourToNodes(md.mesh.x,md.mesh.y,[dInputs,'NoRefinement.exp'],2);
            hminv(find(thickness<20 | flags)) = rsmin1; %nodes where we do not refine
            % Call BAMG
            md = bamg(md,'field',vel,'err',17,'gradation',1.3,'hminVertices',hminv,'hmaxVertices',hmaxv);
            md.private.bamg = struct();     
    end

    % Velocities
    velx = InterpFromMeshToMesh2d(mdcoarse.mesh.elements,mdcoarse.mesh.x,mdcoarse.mesh.y,...
        mdcoarse.results.TransientSolution(end).Vx,md.mesh.x,md.mesh.y);
    vely = InterpFromMeshToMesh2d(mdcoarse.mesh.elements,mdcoarse.mesh.x,mdcoarse.mesh.y,...
        mdcoarse.results.TransientSolution(end).Vy,md.mesh.x,md.mesh.y);
    vel  = sqrt(velx.^2+vely.^2);
    % Thickness
    thickness = InterpFromMeshToMesh2d(mdcoarse.mesh.elements,mdcoarse.mesh.x,mdcoarse.mesh.y,...
        mdcoarse.results.TransientSolution(end).Thickness,md.mesh.x,md.mesh.y);
    hmaxv        = NaN(size(md.mesh.x));
    hminv        = NaN(size(md.mesh.x));
    % Set min/max resolutions
    pos = find(vel<velcat(1));
    hminv(pos) = rsmin1;
    hmaxv(pos) = rsmax1;
    pos = find(vel>=velcat(1) & vel<velcat(2));
    hminv(pos) = rsmin2;
    hmaxv(pos) = rsmax2;
    pos = find(vel>=velcat(2) & vel<velcat(3));
    hminv(pos) = rsmin3;
    hmaxv(pos) = rsmax3;
    pos = find(vel>=velcat(4));
    hminv(pos) = rsmin4;
    hmaxv(pos) = rsmax4;
    % Low resolution flags from Helene
    flags        = ContourToNodes(md.mesh.x,md.mesh.y,[dInputs,'NoRefinement.exp'],2);
    hminv(find(thickness<20 | flags)) = rsmin1; %nodes where we do not refine
    % Refinement of calving glaciers
    flagsRefined = getExpRefinedGlaciers(md.mesh.x,md.mesh.y);
    hmaxv(flagsRefined) = resRefined;
    hminv(flagsRefined) = resRefined;
    % Re-meshing
    md = bamg(md,'field',vel,'err',17,'gradation',1.3,'hminVertices',hminv,'hmaxVertices',hmaxv);
    md.private.bamg = struct();
    [md.mesh.lat,md.mesh.long] = xy2ll(md.mesh.x,md.mesh.y,+1,45,70);
    md.mesh.epsg = 3413;
    if savingmodels
        save([dSaves,'meshmd.mat'],'md','-v7.3');
    end
end


%%% Parameterization %%%
if(param)
    md       = loadmodel([dSaves,'meshmd.mat']);
    % Velocities
    md.initialization.vx = InterpFromMeshToMesh2d(mdcoarse.mesh.elements,mdcoarse.mesh.x,mdcoarse.mesh.y,...
        mdcoarse.results.TransientSolution(end).Vx,md.mesh.x,md.mesh.y);
    md.initialization.vy = InterpFromMeshToMesh2d(mdcoarse.mesh.elements,mdcoarse.mesh.x,mdcoarse.mesh.y,...
        mdcoarse.results.TransientSolution(end).Vy,md.mesh.x,md.mesh.y);
    md.initialization.vel  = sqrt(md.initialization.vx.^2+md.initialization.vy.^2);
    
    % Bed topo from BedMachine
    md.geometry.bed       = interpBedmachineGreenland(md.mesh.x,md.mesh.y,'bed');
    % Surface
    md.geometry.surface   = InterpFromMeshToMesh2d(mdcoarse.mesh.elements,mdcoarse.mesh.x,mdcoarse.mesh.y,...
        mdcoarse.results.TransientSolution(end).Surface,md.mesh.x,md.mesh.y);
    % Remove negative surfaces
    md.geometry.surface = max(md.geometry.surface,md.geometry.bed+md.masstransport.min_thickness);
    % Out of grounded areas: calculate thickness and base of floating ice shelves %%%
    md.geometry.base       = md.geometry.bed;
    di                     = md.materials.rho_ice/md.materials.rho_water;
    pos                    = find(-di/(1-di)*md.geometry.surface > md.geometry.bed);
    md.geometry.base(pos)  = di/(di-1)*md.geometry.surface(pos);
    md.geometry.thickness  = md.geometry.surface-md.geometry.base;
    md.mask.ocean_levelset = md.geometry.thickness+md.geometry.bed/di;
    % Basal forcings
    md.basalforcings.groundedice_melting_rate = zeros(md.mesh.numberofvertices,1);
    md.basalforcings.floatingice_melting_rate = zeros(md.mesh.numberofvertices,1);
    % Interpolate rheology from initialized fine model 
    md.materials.rheology_B = InterpFromMeshToMesh2d(mdinitfine.mesh.elements,mdinitfine.mesh.x,mdinitfine.mesh.y,...
        mdinitfine.materials.rheology_B,md.mesh.x,md.mesh.y);
    % Interpolate friction from initialized fine model 
    md.friction.coefficient = InterpFromMeshToMesh2d(mdinitfine.mesh.elements,mdinitfine.mesh.x,mdinitfine.mesh.y,...
        mdinitfine.friction.coefficient,md.mesh.x,md.mesh.y);
    md.friction.p           = 1*ones(md.mesh.numberofelements,1);
    md.friction.q           = 1*ones(md.mesh.numberofelements,1);
    % Stressbalance BCs
    md.stressbalance.spcvx = NaN*ones(md.mesh.numberofvertices,1);
    md.stressbalance.spcvy = NaN*ones(md.mesh.numberofvertices,1);
    md.stressbalance.spcvz = NaN*ones(md.mesh.numberofvertices,1); % 2D run
    md.stressbalance.referential  = NaN*ones(md.mesh.numberofvertices,6); %normal to bed slope
    md.stressbalance.loadingforce = 0*ones(md.mesh.numberofvertices,3); %changes to gravity
    pos = find((md.mask.ice_levelset<0).*(md.mesh.vertexonboundary)); %ice-covered vertices at domain boundaries (none)
    md.stressbalance.spcvx(pos) = md.initialization.vx(pos);
    md.stressbalance.spcvy(pos) = md.initialization.vy(pos);
    % Mass transport BCs
    md.masstransport.spcthickness = NaN*ones(md.mesh.numberofvertices,1);
    % Flow equation
    md = setflowequation(md,'SSA','all');
    % Pressure
    md.initialization.pressure = InterpFromMeshToMesh2d(mdcoarse.mesh.elements,mdcoarse.mesh.x,mdcoarse.mesh.y,...
        mdcoarse.results.TransientSolution(end).Pressure,md.mesh.x,md.mesh.y);
    % Ice levelset
    md.mask.ice_levelset = InterpFromMeshToMesh2d(mdinitfine.mesh.elements,mdinitfine.mesh.x,mdinitfine.mesh.y,...
        mdinitfine.mask.ice_levelset,md.mesh.x,md.mesh.y);
    levelsetnew          = reinitializelevelset(md,md.mask.ice_levelset);
    md.mask.ice_levelset = levelsetnew;
    % GL migration
    md.groundingline.migration = 'SubelementMigration';
    % SMB from CoarseSpin but adjust basin IDs
    basins            = csvread(BasinsRefinedGrid); %Basin ID numbers (per element)
    md.smb            = mdcoarse.smb;
    md.smb.basin_id   = basins;
    
    if(savingmodels)
        save([dSaves,'initParameterizedmd.mat'],'md','-v7.3');
    end
end

