
addpath %add path to ISSM bin/ and lib/ directories

% Cluster options
optionloadonly       = 0;
outname0             = 'myRunEnsp1';
codepath             = ''; %path to directory with ISSM binaries
execpath             = ''; %path to directory for simulation execution
coresrequested       = 24;
memrequested         = 5; %GB
timerequested        = ; %minutes

nglac                = 11;
nsamples             = 1;
CalvingEnsValsp1_pc1 = zeros(nsamples,nglac);
boundslow            = [240,1100,5,5,4400,2050,2100,990,600,2030,0];
boundsup             = [240,1100,5,5,4400,2050,2100,990,600,2030,50];

for ss=[1:nsamples]
    for ii=[1:nglac]
        CalvingEnsValsp1_pc1(ss,ii) = boundslow(ii)+(boundsup(ii)-boundslow(ii))*rand(1);
        CalvingEnsValsp1_pc1(ss,ii) = round(CalvingEnsValsp1_pc1(ss,ii));
    end
end 
if(optionloadonly==0) 
   csvwrite('./Inputs/CalvingEnsValsp1_pc1.csv',CalvingEnsValsp1_pc1);
end

%%% Inputs %%%
pathInitParamRefinedGrid = './Inputs/initParameterizedmd02May2022.mat';
pathBasinsRefinedGrid    = './Inputs/gridrefinement29April2022basinsGrISzwally.csv';

%%% Run settings %%%
deltat      = 7/365;
endtime     = 1048.0;
reqoutputs  = {'default','MaskIceLevelset','IceVolume','GroundedArea','CalvingCalvingrate','TotalCalvingFluxLevelset','TotalSmb','TotalFloatingBmb','CalvingMeltingrate','TotalCalvingMeltingFluxLevelset'};
outputfreq  = 2600;
lvlReinit   = 1;

%%% Interpolate surface on calving grid %%%
md     = loadmodel(pathInitParamRefinedGrid);

%%% Frontal forcings %%%
basins = csvread(pathBasinsRefinedGrid); %Basin ID numbers (per element)
asl = find(md.geometry.bed>0);
bsl = find(md.geometry.bed<=0);
md.frontalforcings = frontalforcingsrignotautoregression();
md.frontalforcings.num_basins = numel(unique(basins));
md.frontalforcings.basin_id   = basins;
md.frontalforcings.beta0      = [2.0,2.0,2.0,2.0,...
                                 2.5,2.5,2.5,2.5,2.5,...
                                 6.5,6.5,6.5,...
                                 6.5,6.5,6.5,...
                                 4.5,4.5,...
                                 3.5,3.5]; %approx long-term means from Wood et al. (2021)
md.frontalforcings.beta1          = zeros(1,md.frontalforcings.num_basins);
md.frontalforcings.ar_order       = 1;
md.frontalforcings.ar_initialtime = 0;
md.frontalforcings.ar_timestep    = 1;
md.frontalforcings.phi            = zeros(md.frontalforcings.num_basins,1);
md.frontalforcings.subglacial_discharge = zeros(md.mesh.numberofvertices,1);

%%% Levelset %%%
levelsetnew                           = reinitializelevelset(md,md.mask.ice_levelset);
levelsetvals                          = levelsetnew;
flagsRefined                          = getExpRefinedGlaciers(md.mesh.x,md.mesh.y);
levelsetvals(flagsRefined)            = NaN;
md.levelset.spclevelset               = levelsetvals;
md.mask.ice_levelset                  = levelsetnew; %reset mask.ice_levelset
md.levelset.kill_icebergs             = 1;
md.levelset.migration_max             = 1000.0;
md.levelset.reinit_frequency          = lvlReinit;

%%% Initialize calving rates %%%
md.calving.calvingrate = 0*ones(md.mesh.numberofvertices,1);
indsSepRefined         = getExpSepRefinedGlaciers(md.mesh.x,md.mesh.y);

%%% Transient parameters %%%
md.settings.output_frequency   = outputfreq;
md.timestepping.start_time     = 0;
md.timestepping.time_step      = deltat;
md.timestepping.final_time     = endtime;
md.transient.requested_outputs = reqoutputs;
md.results                     = {}; %remove any old results
md.transient.ismovingfront     = 1; %allow for moving front
md.transient.isgroundingline   = 1; %allow for moving GL (i.e. moving mask.ocean_levelset)
md.transient.isstressbalance   = 1;
md.transient.ismasstransport   = 1;
md.transient.issmb             = 1;
md.transient.isthermal         = 0;
md.inversion.iscontrol         = 0;

%%% Parallelization %%%
md.settings.waitonlock = 0;
md.cluster = pace('np',coresrequested,'login','loginname','codepath',codepath,'executionpath',execpath,'mem',memrequested,'time',timerequested);
for ii=[1:numel(CalvingEnsValsp1_pc1(:,1))]
    %%% Calving adjustment per glacier %%%
    ClvGl = CalvingEnsValsp1_pc1(ii,:);
    disp(ClvGl);
    for gg=1:numel(indsSepRefined)
        md.calving.calvingrate(indsSepRefined{gg}) = ClvGl(gg);
    end
    md.calving.calvingrate(asl) = 0;

    md.miscellaneous.name = [outname0,'_pc1calvensvals',num2str(ii)]; %Run name
    md = solve(md,'Transient','runtimename',false,'loadonly',optionloadonly);
    if(optionloadonly==1)
	md = loadresultsfromcluster(md);
	savename = [execpath,outname0,'_pc1calvensvals',num2str(ii),'.mat'];
    	save(savename,'md','-v7.3') %save md
        %rmdir([execpath,outname0,'_pc1calvensvals',num2str(ii)],'s'); %clean-up
    end
end

if(isfile('s1init.mat'))
    delete 's1init.mat';
end

