
addpath %add path to ISSM bin/ and lib/ directories

% Cluster options
optionloadonly       = 0;
outname0             = 'myCoarseSpinp1';
codepath             = ''; %path to directory with ISSM binaries
execpath             = ''; %path to directory for simulation execution
coresrequested       = 20;
memrequested         = 5; %GB
timerequested        = 48*60; %minutes

%%% Load %%%
pathGrIS   = './Inputs/finalInitmdCoarsening.mat';
md         = loadmodel(pathGrIS);
pathBasins = './Inputs/gridcoarsening05April2022basinsGrISzwally.csv';
basins     = csvread(pathBasins); %Basin ID numbers (per element)
pathLapse    = './Inputs/lapseratevals_regressionRacmoGrIS19April2022.csv';
pathElevbins = './Inputs/elevbins_regressionRacmoGrIS19April2022.csv';
pathMeanSMBdata = ['./Inputs/meansElevAndSMB_regressionRacmoGrIS19April2022.csv'];

%%% Run settings %%%
deltat      = 7/365;
endtime     = 5000.0;
reqoutputs  = {'default','IceVolume','GroundedArea','TotalSmb','TotalFloatingBmb'};
outputfreq  = 5200;

%%% SMB implementation %%%
% Average SMB per basin
smbSpin0          = md.smb.mass_balance;
smbSpin           = averaging(md,smbSpin0,5);
md.smb            = SMBautoregression();
md.smb.num_basins = numel(unique(basins));
md.smb.basin_id   = basins;
md.smb.beta1      = zeros(1,md.smb.num_basins);
md.smb.ar_order   = 1;
md.smb.phi        = zeros(md.smb.num_basins,1);
md.smb.ar_initialtime = 0;
md.smb.ar_timestep    = 1;
datasmbmean           = csvread(pathMeanSMBdata);
md.smb.beta0          = datasmbmean(:,2)';
md.smb.refelevation   = datasmbmean(:,1)';
md.smb.lapserates     = csvread(pathLapse);
md.smb.elevationbins  = csvread(pathElevbins);

%No moving front

md = setflowequation(md,'SSA','all');
%%% Boundary conditions %%%
md.masstransport.spcthickness = NaN*ones(md.mesh.numberofvertices,1);
md.stressbalance.spcvx        = NaN*ones(md.mesh.numberofvertices,1);
md.stressbalance.spcvy        = NaN*ones(md.mesh.numberofvertices,1);
md.stressbalance.spcvz        = NaN*ones(md.mesh.numberofvertices,1);

%%% Transient parameters %%%
md.settings.output_frequency   = outputfreq;
md.timestepping.start_time     = 0;
md.timestepping.time_step      = deltat;
md.timestepping.final_time     = endtime;
md.transient.requested_outputs = reqoutputs;
md.transient.ismovingfront     = 0; %no moving front
md.transient.isgroundingline   = 1; %allow for moving GL (i.e. moving mask.ocean_levelset)
md.transient.isstressbalance   = 1;
md.transient.ismasstransport   = 1;
md.transient.issmb             = 1;
md.transient.isthermal         = 0;
md.inversion.iscontrol         = 0;

%%% Solving %%%
md.settings.waitonlock = 0;
md.cluster = pace('np',coresrequested,'login','loginname','codepath',codepath,'executionpath',execpath,'mem',memrequested,'time',timerequested);
md.miscellaneous.name = [outname0,'_pc1']; %Run name
md = solve(md,'Transient','runtimename',false,'loadonly',optionloadonly);
if(optionloadonly==1)
    md = loadresultsfromcluster(md);
    savename = [execpath,outname0,'_pc1.mat'];
    save(savename,'md','-v7.3') %save md
    %rmdir([execpath,outname0,'_pc1'],'s'); %clean-up
end


if(isfile('s1init.mat'))
    delete 's1init.mat';
end

