

% Cluster options
optionloadonly       = 0;
outname0             = 'myCoarseSpinp5';
codepath             = ''; %path to directory with ISSM binaries
execpath             = ''; %path to directory for simulation execution
coresrequested       = 20;
memrequested         = 5; %GB
timerequested        = 48*60; %minutes

%%% Load %%%
pathGrIS   = './Inputs/myCoarseSpinp4pc1_dt7tm20000.mat';
md         = loadmodel(pathGrIS);
pathBasins = './Inputs/gridcoarsening05April2022basinsGrISzwally.csv';
basins     = csvread(pathBasins); %Basin ID numbers (per element)
pathLapse    = './Inputs/lapseratevals_regressionRacmoGrIS19April2022.csv';
pathElevbins = './Inputs/elevbins_regressionRacmoGrIS19April2022.csv';
pathMeanSMBdata = ['./Inputs/meansElevAndSMB_regressionRacmoGrIS19April2022.csv'];

%%% Run settings %%%
deltat      = 7/365;
endtime     = 5000.0;
nproc       = 20;
reqoutputs  = {'default','IceVolume','GroundedArea','TotalSmb','TotalFloatingBmb'};
outputfreq  = 5200;

%%% Set initial conditions %%%
md.geometry.thickness        = md.results.TransientSolution(end).Thickness;
md.initialization.vx         = md.results.TransientSolution(end).Vx;
md.initialization.vy         = md.results.TransientSolution(end).Vy;
md.initialization.vel        = md.results.TransientSolution(end).Vel;
md.mask.ocean_levelset       = md.results.TransientSolution(end).MaskOceanLevelset;
md.initialization.pressure   = md.results.TransientSolution(end).Pressure;
md.geometry.base             = md.results.TransientSolution(end).Base;
md.geometry.surface          = md.results.TransientSolution(end).Surface;
levelsetnew                  = reinitializelevelset(md,md.mask.ice_levelset);
md.mask.ice_levelset         = levelsetnew;

%%% Transient parameters %%%
md.settings.output_frequency   = outputfreq;
md.timestepping.start_time     = 0;
md.timestepping.time_step      = deltat;
md.timestepping.final_time     = endtime;
md.transient.requested_outputs = reqoutputs;
md.results                     = {}; %erase results from old model
md.transient.ismovingfront     = 0; %no moving front
md.transient.isgroundingline   = 1; %allow for moving GL (i.e. moving mask.ocean_levelset)
md.transient.isstressbalance   = 1;
md.transient.ismasstransport   = 1;
md.transient.issmb             = 1;
md.transient.isthermal         = 0;
md.inversion.iscontrol         = 0;

%%% Solving %%%
md.settings.waitonlock = 0;
md.cluster = pace('np',coresrequested,'login','loginname','codepath',codepath,'executionpath',execpath,'mem',memrequested,'time',timerequested);
md.miscellaneous.name = [outname0,'_pc1']; %Run name
disp(md.smb.beta0);
md = solve(md,'Transient','runtimename',false,'loadonly',optionloadonly);
if(optionloadonly==1)
    md = loadresultsfromcluster(md);
    savename = [execpath,outname0,'_pc1.mat'];
    save(savename,'md','-v7.3') %save md
    %rmdir([execpath,outname0,'_pc1'],'s'); %clean-up
end



